#! /bin/env python

import lx
import getopt, re, sys

gls_re = re.compile(r'^%gls:((?:.|\n\s)*)$', re.MULTILINE)
cod_re = re.compile(r'^%cod:((?:.|\n\s)*)$', re.MULTILINE)

id_re = re.compile(r'^([A-Z]+):\t')
alpha_re = re.compile(r'^[a-zA-Z-]+$')
codword_re = re.compile(r'[^\s()]+(?:[(](?:[0-9a-z/]+, )*[0-9a-z/]+[)][^\s()]*)?')
sound_re = re.compile(r'\025[^\025]+\025')

def remove_sound(s):
    mo = sound_re.search(s)
    if mo:
        return s[0:mo.start()]+s[mo.end():]
    else:
        return s
    
def write_morph_utterance(inf, outf, errf=None, split_flag=False):
    records = file(filename, 'rU').read().split('\n*')
    lineno = records[0].count('\n')+2
    for record in records[1:]:
        lno = lineno
        lineno += record.count('\n')+1
        id_mo = id_re.search(record)
        if not id_mo and errf:
            errf.write("## %s:%s, no speaker id:\n*%s\n\n"%(filename,lno,record))
        gls_mo = gls_re.search(record)
        cod_mo = cod_re.search(record)
        if (not gls_mo) and errf:
            errf.write("## %s:%s, missing gls field:\n*%s\n\n"%(filename,lno,record))
        if (not cod_mo) and errf:
            errf.write("## %s:%s, missing cod field:\n*%s\n\n"%(filename,lno,record))
        if (not gls_mo) or (not cod_mo):
            continue
        glss = remove_sound(gls_mo.group(1).lower()).split()
        cods = [mo.group(0) for mo in codword_re.finditer(remove_sound(cod_mo.group(1)))]
        if len(glss) != len(cods):
            if errf:
                errf.write("## %s:%s, gls has %s words, cod has %s words.\n*%s\n\n" %
                           (filename, lno, len(glss), len(cods), record))
            continue
        mismatch_flag = False
        glsscods = zip(glss,cods)
        for (gls,cod) in glsscods:
            gmorphs = gls.split('-')
            cmorphs = cod.split('-')
            if len(gmorphs) != len(cmorphs):
                if errf:
                    errf.write("## %s:%s, gls word %s and cod word %s morphemes don't match.\n*%s\n\n" %
                               (filename, lno, gls, cod, record))
                mismatch_flag = True
                break
            if "" in gmorphs:
                if errf:
                    errf.write("## %s%s, gls word %s has null morpheme.\n*%s\n\n" %
                               (filename, lno, gls, record))
                mismatch_flag = True
                break
        if mismatch_flag:
            continue
        if not id_mo or id_mo.group(1) == "CHI":
            continue
        filtered_glss = [w for w in glss if alpha_re.match(w)]
        if split_flag:
            gmorphs = [m for w in filtered_glss for m in w.split('-')]
            outf.write(' '.join(''.join(gmorphs)))
        else:
            outf.write(' '.join(filtered_glss))
        outf.write(' $$$\n')
            

if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], "s", ["split"])
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)
    affix = ""
    split_flag = False
    for o, a in opts:
        if o in ("-s", "--split"):
            split_flag = True
        else:
            sys.stderr.write("** Can't parse command line option %s\n"%o)
            sys.exit(2)
    errf = file('sesotho.errors', 'w')
    for filename in sorted(lx.findfiles("Sesotho", re.compile(r".*\.cha"))):
        write_morph_utterance(file(filename, "rU"), sys.stdout, errf, split_flag)
